// dhtsensor.cpp created for project NetArduino on 11/08/2011 09:14:16
/* =====================================================================
 * NetArduino for showing weather forecast and time from the internet
 * ---------------------------------------------------------------------
 * dhtsensor.cpp
 *   Class to wrap DHT22 sensor that measures in-door temperature and
 * 	 humidity.
 * =====================================================================
 * By: Zakie Mashiah
 * =====================================================================
 * Copyright
 *  You may use this code with single condition of giving visible
 *  credit to the author.
 * =====================================================================
 * TODO
 * =====================================================================
 *  Revision History
 *      09-Nov-2011 v0.95
 * =====================================================================
 */
#include <avr/pgmspace.h>
#include <WProgram.h>
#include <dht.h>

#include "utilfuncs.h"
#include "dhtsensor.h"

#define PIN_DHT22_SENSOR	3
static dht dht22;

float dhtSensor::getTemperature() { return dht22.temperature; }
float dhtSensor::getHumidity() { return dht22.humidity; }


void dhtSensor::init()
{
	lastRead = 0;
}


void dhtSensor::read()
{
	int dht22Error;
	bool cont = true;
	int i = 0;
  
	for (i=0, cont = true; cont && (i<3); i++)
	{
		dht22Error = dht22.read22(PIN_DHT22_SENSOR);
		if ( dht22Error == 0)
		{
			cont = false;
			lastRead = millis();
		}
		else
		{
			delay(2100);
			Console_printP(PSTR(">>> DHT22 Error: "));
			Console.print(dht22Error, DEC);
			Console_printlnP(PSTR(" <<<"));
		}
	}
}

void dhtSensor::print()
{
	unsigned long __now = millis();
	
	if ((__now - lastRead) > 2000) // Read only once in 2 seconds
		read();
	Console_printP(PSTR("Indoor Temp: "));
	Console.print(getTemperature());
	Console_printP(PSTR("*C Humidity: "));
	Console.print(getHumidity());
	Console_printlnP(PSTR("%"));
}

void dhtSensor::send()
{
	char buff[10];
	int temp;
	
	temp = (getTemperature() * 10.0);
	sprintf(buff, "DT%3d", temp);
	sendReading(buff, strlen(buff));
	temp = (getHumidity() * 10.0);
	sprintf(buff, "DH%d", temp);
	sendReading(buff, strlen(buff));
}

void dhtSensor::sendReading(char *e, int l)
{
	char *ptr = e;
	int i;

	serialDisplay.print((char)'[');
	for(i=0; i<l; i++, ptr++)
		serialDisplay.print( *ptr );
	serialDisplay.print((char)']');
}
